/* This file is part of GPump, a Pump.io client.
 *
 * GPump (THE SOFTWARE) is made available under the terms and conditions of the
 * GNU Lesser General Public Licence 3.0. A copy of the licence can be read
 * in the file lgpl-3.0.txt in the root of this project.
 */

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;

const Lang = imports.lang;
const System = imports.system;

let _default_settings_object = null;

/**
 * Please use this function when getting the settings data object. It is not
 * good to 
 */
function get_settings () {
  if (!_default_settings_object) {
    _default_settings_object = new SettingsData ();
  }
  return _default_settings_object;
}
/**
 * NO NOT CREATE INSTANCES FROM THIS OBJECT, USE THE get_settings () FUNCTION!
 */
const SettingsData = new Lang.Class ({
  Name: 'SettingsData',
  
  /* Member definitions */
  _settings_file_path: String,
  _settings: Array,
  _settings_file: Gio.File,
  
  _init: function () {
    /* First we construct the path for where to store the settings file. */
    this._settings_file_path = "";
    this._settings_file_path += GLib.get_user_config_dir ();
    this._settings_file_path += "/gpump/gpump.json";
    print ("Config file is: " + this._settings_file_path);
    
    this._settings_file = Gio.File.new_for_path (this._settings_file_path);
    
    /* Then we check that the file exists. If the file doen not ekist we
     * construct some sane default values.
     */
    if (!GLib.file_test (this._settings_file_path, GLib.FileTest.EXISTS)) {
      /* -1 is undefined, and will be set at construction of the window */
      this._settings = {
        ui: {
          x: -1,
          y: -1,
          h: 500,
          w: 500
        },
        main: {
          privacy: {
            show_full_name: true,
            only_show_avatar: false
          },
          use_dark: false,
          first_run: true
        }
      };
      // DEBUG:
      print (JSON.stringify (this._settings, null, 2).toString () );
      
      let file_stream = this._settings_file.create (Gio.FileCreateFlags.PRIVATE,
                                                    null);
      file_stream.write_all (JSON.stringify (
          this._settings, null, 2).toString (), null);
      
      file_stream.close (null);
      
    } else {
      /* The file exists, we load the settings data into memory */
      let file_stream = this._settings_file.read (null);
      
      /* See: http://stackoverflow.com/a/21146281
       */
      let file_info = this._settings_file.query_info("standard::size",
                                             Gio.FileQueryInfoFlags.NONE, null);
      let size = file_info.get_size ();
      
      let buffer = file_stream.read_bytes (size, null).get_data ();
      
      this._settings = JSON.parse (buffer);
      
      //DEBUG:
      print (JSON.stringify (this._settings, null, 2).toString () );
      
    }
  },
  
  /**
   * Sets a value in the setting object.
   * 
   * return: false on fail
   * return: true when everything is OK.
   */
  set_setting: function (setting, value) {
    if (typeof setting != "string") {
      print ("ERROR: The \"setting\" paramenter must be a string... Exiting.");
      return false;
    }
    
  },
  
  /**
   * Gets a value from the settings object.
   *
   * returns a complex object with two fealds: ok and data.
   *
   * If ok is false something went wrong and the data feald will be undefined.
   *
   * If ok is true everything is ok and the data feald will be set to the value.
   */
  get_setting: function (setting) {
    let ret_data = {
      ok: true,
      data: undefined
    };
    
    return ret_data;
  },
  
  /**
   * Commits changes to the settings object to file.
   */
  commit_to_file: function () {
    
  }
  
});



